library dashstrap_iconbutton;

import 'dart:math';

import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

enum DSIconButtonSize { small, normal, large }
enum DSIconButtonFeedbackStyle { normal, opacity }
enum DSIconButtonStyle { normal, outline }
enum DSIconButtonShape { round, square }

// ignore: must_be_immutable
class DSIconButton extends StatefulWidget {
  final IconData icon;
  final Color? color;
  final Color iconColor;
  final DSIconButtonStyle style;
  final DSIconButtonSize size;
  final DSIconButtonFeedbackStyle feedbackStyle;
  final Function(TapUpDetails)? onTapUp;
  final Function(TapDownDetails)? onTapDown;
  final double activeOpacity;
  final bool disabled;
  final Color? disabledColor;
  final DSIconButtonShape shape;
  final double? iconSize;
  bool _isFab = false;

  DSIconButton.rounded({
    required this.icon,
    this.color,
    this.iconColor = Colors.white,
    this.style = DSIconButtonStyle.normal,
    this.size = DSIconButtonSize.normal,
    this.feedbackStyle = DSIconButtonFeedbackStyle.normal,
    this.onTapDown,
    this.onTapUp,
    this.activeOpacity = 0.3,
    this.disabled = false,
    this.disabledColor,
    this.iconSize,
    this.shape = DSIconButtonShape.round,
    Key? key,
  }) : super(key: key);

  DSIconButton.square({
    required this.icon,
    this.color,
    this.iconColor = Colors.white,
    this.style = DSIconButtonStyle.normal,
    this.size = DSIconButtonSize.normal,
    this.feedbackStyle = DSIconButtonFeedbackStyle.normal,
    this.onTapDown,
    this.onTapUp,
    this.activeOpacity = 0.3,
    this.disabled = false,
    this.disabledColor,
    this.iconSize,
    this.shape = DSIconButtonShape.square,
    Key? key,
  }) : super(key: key);

  DSIconButton.fab({
    required this.icon,
    this.color,
    this.iconColor = Colors.white,
    this.style = DSIconButtonStyle.normal,
    this.size = DSIconButtonSize.normal,
    this.feedbackStyle = DSIconButtonFeedbackStyle.normal,
    this.onTapDown,
    this.onTapUp,
    this.activeOpacity = 0.3,
    this.disabled = false,
    this.disabledColor,
    this.iconSize,
    this.shape = DSIconButtonShape.round,
    Key? key,
  }) : super(key: key) {
    _isFab = true;
  }

  DSIconButton({
    required this.icon,
    this.color,
    this.iconColor = Colors.white,
    this.style = DSIconButtonStyle.normal,
    this.size = DSIconButtonSize.normal,
    this.feedbackStyle = DSIconButtonFeedbackStyle.normal,
    this.onTapDown,
    this.onTapUp,
    this.activeOpacity = 0.3,
    this.disabled = false,
    this.disabledColor,
    this.iconSize,
    this.shape = DSIconButtonShape.round,
    Key? key,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _isFab ? _DSIconButtonState.fab() : _DSIconButtonState();
  }
}

class _DSIconButtonState extends State<DSIconButton> {
  late Color defaultBackgroundColor;
  late Color currentBackgroundColor;
  late double defaultOpacity;
  late double currentOpacity;
  double elevation = 0;

  _DSIconButtonState();

  _DSIconButtonState.fab() {
    elevation = 8;
  }

  @override
  void didChangeDependencies() {
    defaultBackgroundColor = getBackgroundColor();
    currentBackgroundColor = defaultBackgroundColor;
    defaultOpacity = 1;
    currentOpacity = 1;
    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTapDown: onTapDown,
      onTapUp: onTapUp,
      child: AnimatedOpacity(
        opacity: currentOpacity,
        duration: Duration(milliseconds: 150),
        child: Material(
          shadowColor: Colors.black,
          shape: CircleBorder(),
          elevation: elevation,
          child: AnimatedContainer(
            padding: EdgeInsets.all(getButtonPadding()),
            duration: Duration(milliseconds: 200),
            decoration: BoxDecoration(
              color: widget.style == DSIconButtonStyle.outline
                  ? Colors.transparent
                  : currentBackgroundColor,
              borderRadius: BorderRadius.circular(getBorderRadius()),
              border: Border.all(
                width: widget.style == DSIconButtonStyle.outline ? 1 : 0,
                color: widget.style == DSIconButtonStyle.outline
                    ? currentBackgroundColor
                    : Colors.transparent,
              ),
            ),
            child: Icon(
              widget.icon,
              size: getIconSize(),
              color: getIconColor(),
            ),
          ),
        ),
      ),
    );
  }

  Color getIconColor() {
    if (widget.disabled) {
      if (widget.style == DSIconButtonStyle.outline) {
        return widget.disabledColor ?? Color(0xFFEAEAEA);
      }

      return Colors.white;
    }

    if (widget.style == DSIconButtonStyle.outline) {
      return currentBackgroundColor;
    }

    return widget.iconColor;
  }

  double getIconSize() {
    if (widget.iconSize != null) {
      return widget.iconSize!;
    }

    switch (widget.size) {
      case DSIconButtonSize.small:
        return 18;
      case DSIconButtonSize.normal:
        return 22;
      case DSIconButtonSize.large:
        return 26;
    }
  }

  double getBorderRadius() {
    if (widget.shape == DSIconButtonShape.square) {
      return 4;
    }

    return (getIconSize() + (getButtonPadding() * 2)) / 2;
  }

  Color getBackgroundColor() {
    if (widget.disabled) {
      return widget.disabledColor ?? Color(0xFFEAEAEA);
    }

    if (widget.color != null) {
      return widget.color!;
    }

    return Theme.of(context).accentColor;
  }

  double getButtonPadding() {
    double padding = 8;
    switch (widget.size) {
      case DSIconButtonSize.small:
        break;
      case DSIconButtonSize.normal:
        padding = padding * 1.5;
        break;
      case DSIconButtonSize.large:
        padding = padding * 2;
        break;
    }

    return padding;
  }

  void onTapDown(TapDownDetails tapDownDetails) {
    if (widget.disabled) {
      return;
    }

    Color activeColor = Color.fromARGB(
      defaultBackgroundColor.alpha,
      max(defaultBackgroundColor.red - 30, 0),
      max(defaultBackgroundColor.green - 30, 0),
      max(defaultBackgroundColor.blue - 30, 0),
    );

    if (widget.feedbackStyle == DSIconButtonFeedbackStyle.opacity) {
      setState(() {
        currentOpacity = widget.activeOpacity;
      });
    } else {
      setState(() {
        currentBackgroundColor = activeColor;
      });
    }

    if (widget.onTapDown != null) {
      widget.onTapDown!(tapDownDetails);
    }
  }

  void onTapUp(TapUpDetails tapUpDetails) {
    if (widget.disabled) {
      return;
    }
    setState(() {
      currentOpacity = defaultOpacity;
      currentBackgroundColor = defaultBackgroundColor;
    });

    if (widget.onTapUp != null) {
      widget.onTapUp!(tapUpDetails);
    }
  }
}
