import 'dart:math';

import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';

import 'package:dashstrap_iconbutton/dashstrap_iconbutton.dart';

void main() {
  Color _getActiveColor(Color baseColor) {
    int red = max(baseColor.red - 30, 0);
    int green = max(baseColor.green - 30, 0);
    int blue = max(baseColor.blue - 30, 0);

    return Color.fromRGBO(red, green, blue, 1);
  }

  testWidgets("Test that default button renders correctly", (tester) async {
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSIconButton(
            icon: Icons.add,
            color: Color(0xFF1890FF),
          ),
        ),
      ),
    );

    Finder gestureFinder = find.byType(GestureDetector);
    Finder opacityFinder = find.byType(AnimatedOpacity);
    Finder containerFinder = find.byType(AnimatedContainer);
    Finder iconFinder = find.byType(Icon);

    // Check that the right widgets render the right number of times
    expect(gestureFinder, findsOneWidget);
    expect(opacityFinder, findsOneWidget);
    expect(iconFinder, findsOneWidget);
    expect(containerFinder, findsNWidgets(1));

    // Check that background of default button is the default color
    AnimatedContainer container =
        containerFinder.last.evaluate().single.widget as AnimatedContainer;
    expect((container.decoration as BoxDecoration).color,
        equals(Color(0xFF1890FF)));

    Icon icon = iconFinder.last.evaluate().single.widget as Icon;
    expect(icon.icon, Icons.add);
  });

  testWidgets("Test that key provided is assigned to the root widget",
      (tester) async {
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSIconButton(
            icon: Icons.add,
            key: Key("TestKey"),
          ),
        ),
      ),
    );

    Finder containerFinder = find.byType(DSIconButton);
    DSIconButton root =
        containerFinder.first.evaluate().single.widget as DSIconButton;
    expect(root.key, equals(Key("TestKey")));
  });

  testWidgets("Test that provided icon color property is applied",
      (tester) async {
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSIconButton(
            icon: Icons.add,
            iconColor: Colors.red,
          ),
        ),
      ),
    );

    Finder iconFinder = find.byType(Icon);
    Icon icon = iconFinder.first.evaluate().single.widget as Icon;
    expect(icon.color, Colors.red);
  });

  testWidgets("Test that on pressing the button the background color changes",
      (tester) async {
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSIconButton(
            icon: Icons.add,
            color: Color(0xFF1890FF),
          ),
        ),
      ),
    );

    await tester.startGesture(tester.getCenter(find.byIcon(Icons.add)));
    await tester.pump();

    Finder animatedContainerFinder = find.byType(AnimatedContainer);
    AnimatedContainer container = animatedContainerFinder.last
        .evaluate()
        .single
        .widget as AnimatedContainer;
    expect((container.decoration as BoxDecoration).color,
        equals(_getActiveColor(Color(0xFF1890FF))));
  });

  testWidgets(
      "Test that for feedback style opacity, on press the opacity changes accordingly",
      (tester) async {
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSIconButton(
            icon: Icons.add,
            feedbackStyle: DSIconButtonFeedbackStyle.opacity,
          ),
        ),
      ),
    );

    await tester.startGesture(tester.getCenter(find.byIcon(Icons.add)));
    await tester.pump();

    Finder animatedContainerFinder = find.byType(AnimatedOpacity);
    AnimatedOpacity container = animatedContainerFinder.last
        .evaluate()
        .single
        .widget as AnimatedOpacity;
    expect(container.opacity, equals(0.3));
  });

  testWidgets(
      "Test that for feedback style opacity, on press the opacity changes accordingly when using custom active opacity",
      (tester) async {
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSIconButton(
            icon: Icons.add,
            feedbackStyle: DSIconButtonFeedbackStyle.opacity,
            activeOpacity: 0.5,
          ),
        ),
      ),
    );

    await tester.startGesture(tester.getCenter(find.byIcon(Icons.add)));
    await tester.pump();

    Finder animatedContainerFinder = find.byType(AnimatedOpacity);
    AnimatedOpacity container = animatedContainerFinder.last
        .evaluate()
        .single
        .widget as AnimatedOpacity;
    expect(container.opacity, equals(0.5));
  });

  testWidgets("Test that tap down and up are called, and in the correct order",
      (tester) async {
    // The widget uses futures, without runAsync the test will throw a pending timer error
    await tester.runAsync(() async {
      List<String> tapResult = [];
      await tester.pumpWidget(
        MaterialApp(
          home: Material(
            child: DSIconButton(
              icon: Icons.add,
              onTapDown: (_) {
                tapResult.add("down");
              },
              onTapUp: (_) {
                tapResult.add("up");
              },
            ),
          ),
        ),
      );

      await tester.tap(find.byIcon(Icons.add));
      await tester.pump();

      expect(tapResult.length, equals(2));
      expect(tapResult[0], equals("down"));
      expect(tapResult[1], equals("up"));
    });
  });

  testWidgets("Test that button color changes properly when disabled",
      (tester) async {
    await tester.runAsync(() async {
      await tester.pumpWidget(
        MaterialApp(
          home: Material(
            child: DSIconButton(
              icon: Icons.add,
              disabled: true,
            ),
          ),
        ),
      );

      Finder containerFinder = find.byType(AnimatedContainer);
      expect(containerFinder, findsOneWidget);

      AnimatedContainer container =
          containerFinder.last.evaluate().single.widget as AnimatedContainer;
      expect((container.decoration as BoxDecoration).color,
          equals(Color(0xFFEAEAEA)));
    });
  });

  testWidgets("Test that custom disabled color is used when disabled",
      (tester) async {
    await tester.runAsync(() async {
      await tester.pumpWidget(
        MaterialApp(
          home: Material(
            child: DSIconButton(
              icon: Icons.add,
              disabled: true,
              disabledColor: Colors.red,
            ),
          ),
        ),
      );

      Finder containerFinder = find.byType(AnimatedContainer);
      expect(containerFinder, findsOneWidget);

      AnimatedContainer container =
          containerFinder.last.evaluate().single.widget as AnimatedContainer;
      expect((container.decoration as BoxDecoration).color, equals(Colors.red));
    });
  });

  testWidgets(
      "Test that for feedback style opacity, on press the opacity does not change when disabled",
      (tester) async {
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSIconButton(
            icon: Icons.add,
            feedbackStyle: DSIconButtonFeedbackStyle.opacity,
            disabled: true,
          ),
        ),
      ),
    );

    await tester.startGesture(tester.getCenter(find.byIcon(Icons.add)));
    await tester.pump();

    Finder animatedContainerFinder = find.byType(AnimatedOpacity);
    AnimatedOpacity container = animatedContainerFinder.last
        .evaluate()
        .single
        .widget as AnimatedOpacity;
    expect(container.opacity, equals(1));
  });

  testWidgets(
      "Test that on pressing the button the background color does not change when disabled",
      (tester) async {
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSIconButton(
            icon: Icons.add,
            disabledColor: Color(0xFF1890FF),
            disabled: true,
          ),
        ),
      ),
    );

    await tester.startGesture(tester.getCenter(find.byIcon(Icons.add)));
    await tester.pump();

    Finder animatedContainerFinder = find.byType(AnimatedContainer);
    AnimatedContainer container = animatedContainerFinder.last
        .evaluate()
        .single
        .widget as AnimatedContainer;
    expect((container.decoration as BoxDecoration).color,
        equals(Color(0xFF1890FF)));
  });

  testWidgets("Test that tap down and up are not called when disabled",
      (tester) async {
    // The widget uses futures, without runAsync the test will throw a pending timer error
    await tester.runAsync(() async {
      List<String> tapResult = [];
      await tester.pumpWidget(
        MaterialApp(
          home: Material(
            child: DSIconButton(
              disabled: true,
              icon: Icons.add,
              onTapDown: (_) {
                tapResult.add("down");
              },
              onTapUp: (_) {
                tapResult.add("up");
              },
            ),
          ),
        ),
      );

      await tester.tap(find.byIcon(Icons.add));
      await tester.pump();

      expect(tapResult.length, equals(0));
    });
  });
}
